package com.hncy.service.wechat.applet.biz.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.common.exception.custom.LocalBusinessException;
import com.hncy.jackson.spring.boot.starter.core.JSONObject;
import com.hncy.service.wechat.applet.biz.entity.*;
import com.hncy.service.wechat.applet.biz.mapper.*;
import com.hncy.service.wechat.applet.biz.service.ComponentDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hncy.service.wechat.applet.biz.service.ComponentsRemoteService;
import com.hncy.service.wechat.applet.biz.utils.FastdfsClientUtil;
import com.hncy.service.wechat.applet.biz.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 部件详情 服务实现类
 * </p>
 *
 * @author cs
 * @since 2021-11-02
 */
@Service
@Slf4j
public class ComponentDetailServiceImpl extends ServiceImpl<ComponentDetailMapper, ComponentDetail> implements ComponentDetailService {

    private final String GARABAGE = "垃圾箱";
    private final String MANHOLE = "井盖";
    private final String ONE = "1";
    private final String TWO = "2";
    private final String COMMUNAL_FACILITIES = "公用设施";
    private final String CITY_APPEARANCE = "市容环境";
    private final String EXTENDS_PARTS = "拓展部件";

    //文件储存ip地址
    @Value("${file.upload}")
    private String basePath;

    @Autowired
    private ComponentsRemoteService componentsRemoteService;
    @Autowired
    private ComponentDetailMapper componentDetailMapper;
    @Autowired
    private ComponentIssuesReportMapper componentIssuesReportMapper;
    @Autowired
    private ComponentIssueReportRelationMapper componentIssueReportRelationMapper;
    @Autowired
    private FastdfsClientUtil fastdfsClientUtil;
    @Autowired
    private ComponentPictureMapper componentPictureMapper;
    @Autowired
    private ComponentPictureRelationMapper componentPictureRelationMapper;
    @Autowired
    private PlatformAuditMapper platformAuditMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean getComponentByCtiySys() {
        List<Map<String, Object>> maps = componentsRemoteService.loadAllComponents();
        for (Map<String, Object> map : maps) {
            String componentLogo = map.get("code").toString();
            String componentName = map.get("name").toString();
            String xCode = map.get("lon").toString();
            String yCode = map.get("lat").toString();
            if(componentName.equalsIgnoreCase("垃圾箱")) {
                ComponentDetail componentDetail = new ComponentDetail();
                componentDetail.setComponentLogo(componentLogo);
                componentDetail.setComponentName(componentName);
                componentDetail.setXCode(xCode);
                componentDetail.setYCode(yCode);
                componentDetail.setComponentTypeName(GARABAGE);
                componentDetail.setComponentTypeCode(ONE);
                componentDetailMapper.insert(componentDetail);
            }else {
                //井盖
                ComponentDetail componentDetail = new ComponentDetail();
                componentDetail.setComponentLogo(componentLogo);
                componentDetail.setComponentName(componentName);
                componentDetail.setXCode(xCode);
                componentDetail.setYCode(yCode);
                componentDetail.setComponentTypeName(MANHOLE);
                componentDetail.setComponentTypeCode(TWO);
                componentDetailMapper.insert(componentDetail);
            }
        }
        return ResultBean.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean getComponentTypeAndRegCondByCtiySys() {
        Map<String, Object> stringObjectMap = componentsRemoteService.queryTypeAndRegCond();
        String s = JSONObject.getInstance().toJSONString(stringObjectMap);
        log.info(s);

/*        //部门大小类
        List<Map<String,String>> typeLists = (List<Map<String, String>>) stringObjectMap.get("typeList");
        for(Map<String,String> typeList : typeLists) {
            String subClassName = typeList.get("subClassName");
            String subClassCode = typeList.get("subClassCode");
            String bigClassName = typeList.get("bigClassName");
            String bigClassCode = typeList.get("bigClassCode");

            List<ComponentDetail> componentDetails = componentDetailMapper.selectBySubClassName(subClassName);
            componentDetails.stream().forEach(componentDetail -> {
                componentDetail.setComponentCode(subClassCode);
                componentDetail.setComponentSuperCode(bigClassCode);
                componentDetail.setComponentSuperName(bigClassName);
                componentDetailMapper.updateById(componentDetail);
            });
        }
        //上报问题案件类型
        List<Map<String,String>> regCondLists = (List<Map<String, String>>) stringObjectMap.get("regCondList");
        for (Map<String,String> regCondList : regCondLists) {
            String regCondCode = regCondList.get("regCondCode");
            String regCondName = regCondList.get("regCondName");
            ComponentIssuesReport componentIssuesReport = new ComponentIssuesReport();
            componentIssuesReport.setRegCondCode(regCondCode);
            componentIssuesReport.setRegCondName(regCondName);
            componentIssuesReportMapper.insert(componentIssuesReport);
            String comIssuesId = componentIssuesReport.getId();

            String subClassCode = regCondList.get("subClassCode");
            String subClassName = regCondList.get("subClassName");
            ComponentDetail componentDetail = componentDetailMapper.selectBySubClassNameAndsubClassCode(subClassCode,subClassName);
            ComponentIssueReportRelation componentIssueReportRelation = new ComponentIssueReportRelation();
            componentIssueReportRelation.setComponentObjId(componentDetail.getId());
            componentIssueReportRelation.setComponentIssuesId(comIssuesId);
            componentIssueReportRelationMapper.insert(componentIssueReportRelation);
        }*/

        return ResultBean.success(s);
    }

    @Override
    public ResultBean<ComponentAcquisitionVo> judgeAIComponentIsExist(String componentTypeName, String xCode, String yCode) {
        ComponentAcquisitionVo componentAcquisitionVo = new ComponentAcquisitionVo();
        List<AIScanComponentsVo> aiScanComponentsVos = new ArrayList<>();
        List<ComponentSubsVo> componentSubsVos = new ArrayList<>();

        List<ComponentDetail> componentDetails = componentDetailMapper.judgeAIComponentIsExist(componentTypeName,xCode,yCode);
        if(CollectionUtil.isEmpty(componentDetails)) {
            throw new LocalBusinessException(430,componentTypeName);
        }
        componentDetails.stream().forEach(componentDetail -> {
            componentAcquisitionVo.setComponentTypeCode(componentDetail.getComponentTypeCode());
            componentAcquisitionVo.setComponentTypeName(componentDetail.getComponentTypeName());
            ComponentSubsVo componentSubsVo = new ComponentSubsVo();
            BeanUtils.copyProperties(componentDetail,componentSubsVo);
            ComponentIssueReportRelation componentIssueReportRelation = componentIssueReportRelationMapper.selectIssueIdByComponentId(componentDetail.getId());
            ComponentIssuesReport componentIssuesReport = componentIssuesReportMapper.selectById(componentIssueReportRelation.getComponentIssuesId());
            componentSubsVo.setRegCondName(componentIssuesReport.getRegCondName());
            componentSubsVo.setRegCondCode(componentIssuesReport.getRegCondCode());
            componentSubsVos.add(componentSubsVo);
        });
        Map<String, List<ComponentSubsVo>> collect = componentSubsVos
                                                    .stream()
                                                    .collect(Collectors.groupingBy(ComponentSubsVo :: getComponentName));
        for(String componentName : collect.keySet()) {
            AIScanComponentsVo aiScanComponentsVo = new AIScanComponentsVo();
            for (ComponentSubsVo componentSubsVo : collect.get(componentName)) {
                aiScanComponentsVo.setComponentName(componentSubsVo.getComponentName());
                aiScanComponentsVo.setComponentCode(componentSubsVo.getComponentCode());
                aiScanComponentsVo.setComponentSubsVoList(collect.get(componentName));
                aiScanComponentsVo.setSum(String.valueOf(collect.get(componentName).size()));
            }
            aiScanComponentsVos.add(aiScanComponentsVo);
        }
        componentAcquisitionVo.setAiScanComponentsVoList(aiScanComponentsVos);
        return ResultBean.success(componentAcquisitionVo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean componentAcquisition(String componentName,
                                           String componentCode,
                                           String componentSuperName,
                                           String componentSuperCode,
                                           String componentTypeName,
                                           String componentTypeCode,
                                           String componentAddress,
                                           String xCode,
                                           String yCode,
                                           LocalDateTime patrolInspectionTime,
                                           String regCondCode,
                                           String regCondName,
                                           String issueDescription,
                                           List<String> pictureIds,
                                           String collectorNickName) {

        //部件详情
        ComponentDetail componentDetail = new ComponentDetail();
        componentDetail.setComponentName(componentName);
        componentDetail.setComponentCode(componentCode);
        componentDetail.setComponentSuperName(componentSuperName);
        componentDetail.setComponentSuperCode(componentSuperCode);
        componentDetail.setComponentTypeName(componentTypeName);
        componentDetail.setComponentTypeCode(componentTypeCode);
        componentDetail.setComponentAddress(componentAddress);
        componentDetail.setXCode(xCode);
        componentDetail.setYCode(yCode);
        componentDetail.setPatrolInspectionTime(patrolInspectionTime);
        componentDetail.setDeleted(0);
        componentDetail.setGmtCreate(DateTime.now());
        componentDetail.setCreateBy(collectorNickName);
        int insert = componentDetailMapper.insert(componentDetail);
        if(insert == 0) {
            throw new BusinessException("部件采集失败，请重新采集");
        }
        String componentId = componentDetail.getId();

        //部件采集与图片关联
        pictureIds.stream().forEach(pictureId ->{
            ComponentPictureRelation componentPictureRelation = new ComponentPictureRelation();
            componentPictureRelation.setComponentDetailId(componentId);
            componentPictureRelation.setComponentPictureId(pictureId);
            componentPictureRelation.setComponentIssuesReportId(null);
            componentPictureRelation.setDeleted(0);
            componentPictureRelation.setGmtCreate(DateTime.now());
            int insert2 = componentPictureRelationMapper.insert(componentPictureRelation);
            if(insert2 == 0) {
                throw new BusinessException("部件采集失败，请重新采集");
            }
        });

        //部件问题上报
        ComponentIssuesReport componentIssuesReport = new ComponentIssuesReport();
        componentIssuesReport.setComponentReportName(componentName);
        componentIssuesReport.setComponentReportCode(componentCode);
        componentIssuesReport.setRegCondName(regCondName);
        componentIssuesReport.setRegCondCode(regCondCode);
        componentIssuesReport.setIssueDescription(issueDescription);
        componentIssuesReport.setDeleted(0);
        componentIssuesReport.setGmtCreate(DateTime.now());
        int insert1 = componentIssuesReportMapper.insert(componentIssuesReport);
        if(insert1 == 0) {
            throw new BusinessException("部件采集失败，请重新采集");
        }
        String issueId = componentIssuesReport.getId();

        //新增关联关系
        ComponentIssueReportRelation componentIssueReportRelation = new ComponentIssueReportRelation();
        componentIssueReportRelation.setComponentIssuesId(issueId);
        componentIssueReportRelation.setComponentObjId(componentId);
        componentIssueReportRelation.setDeleted(0);
        componentIssueReportRelation.setGmtCreate(DateTime.now());
        int insert2 = componentIssueReportRelationMapper.insert(componentIssueReportRelation);
        if(insert2 == 0) {
            throw new BusinessException("部件采集失败，请重新采集");
        }

        //上报给平台端审核
        PlatformAudit platformAudit = new PlatformAudit();
        platformAudit.setComponentName(componentName);
        platformAudit.setComponentDetailId(componentId);
        platformAudit.setComponentStatus(regCondName);
        platformAudit.setComponentCollector(collectorNickName);
        platformAudit.setComponentCollectTime(DateTime.now());
        platformAudit.setAuditStatus("1");
        platformAudit.setAuditTime(DateTime.now());
        platformAudit.setComponentIssuesReportId(issueId);
        platformAudit.setDeleted(0);
        platformAudit.setGmtCreate(DateTime.now());
        int insert3 = platformAuditMapper.insert(platformAudit);
        if(insert3 == 0) {
            throw new BusinessException("部件采集失败，请重新采集");
        }

        return ResultBean.success("部件采集成功");
    }

    @Override
    public ResultBean<PatrolInspectionSumVo> selectTodaySum() {
        PatrolInspectionSumVo patrolInspectionSumVo = new PatrolInspectionSumVo();
        String patrolInspection = componentDetailMapper.selectTodaySum();
        String componentAcquisition = componentDetailMapper.selectTodayComponentsSum();
        String reportIssue = componentIssuesReportMapper.selectTodaySum();
        patrolInspectionSumVo.setPatrolInspectionSum(patrolInspection);
        patrolInspectionSumVo.setComponentAcquisitionSum(componentAcquisition);
        patrolInspectionSumVo.setReportIssueSum(reportIssue);
        return ResultBean.success(patrolInspectionSumVo);
    }

    @Override
    public ResultBean<List<PatrolInspectionComponentsSumVo>> selectTodayComponentsTypeSum() {
        List<PatrolInspectionComponentsSumVo> patrolInspectionComponentsSumVos = new ArrayList<>();
        List<ComponentDetail> componentDetails = componentDetailMapper.selectTodayComponentsTypeSum();
        Map<String,List<ComponentDetail>> map = componentDetails.stream().collect(Collectors.groupingBy(ComponentDetail::getComponentName));
        for(String component : map.keySet()) {
            PatrolInspectionComponentsSumVo patrolInspectionComponentsSumVo = new PatrolInspectionComponentsSumVo();
            patrolInspectionComponentsSumVo.setPatrolInspectionComponentsTypeName(component);
            patrolInspectionComponentsSumVo.setPatrolInspectionComponentsSum(String.valueOf(map.get(component).size()));
            patrolInspectionComponentsSumVos.add(patrolInspectionComponentsSumVo);
        }
        return ResultBean.success(patrolInspectionComponentsSumVos);
    }

    @Override
    public ResultBean savepatrolInspectionTime(LocalDateTime patrolInspectionTime, List<String> componentLogos) {
        componentLogos.stream().forEach(componentLogo ->{
            int insert = componentDetailMapper.savepatrolInspectionTime(patrolInspectionTime,componentLogo);
            if(insert == 0) {
                throw new BusinessException("添加巡检记录失败");
            }
        });
        return ResultBean.success("添加巡检记录成功");
    }
}
